﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Data;
using System.Data.Entity.Infrastructure;
using System.Linq;

namespace DL.Core.Data.EntityFramework.Auditing
{
    public class AuditorBase : IAuditor
    {
        private readonly DateTimeOffset changeTime;

        public AuditorBase()
        {
            this.changeTime = DateTimeOffset.UtcNow;
        }

        public IEnumerable<AuditLog> AuditChange(DbEntityEntry entry, string userToken)
        {
            var result = new List<AuditLog>();

            if (entry.State == EntityState.Added)
            {
                AuditLog log = this.GetAddedAudit(userToken, entry);
                result.Add(log);
            }
            else if (entry.State == EntityState.Deleted)
            {
                AuditLog log = this.GetDeleteAudit(userToken, entry);
                result.Add(log);
            }
            else if (entry.State == EntityState.Modified)
            {
                IEnumerable<AuditLog> logs = this.GetModifiedAudit(userToken, entry);
                result.AddRange(logs);
            }

            return result;
        }

        protected virtual string GetRecordIdFromCurrentValues(DbEntityEntry entry)
        {
            return entry.CurrentValues.GetValue<object>("Id").ToString();
        }

        protected virtual string GetRecordIdFromOriginalValues(DbEntityEntry entry)
        {
            return entry.OriginalValues.GetValue<object>("Id").ToString();
        }

        protected virtual string GetTableName(DbEntityEntry entry)
        {
            var tableAttribute = entry.Entity.GetType()
                .GetCustomAttributes(typeof(TableAttribute), false)
                .SingleOrDefault() as TableAttribute;

            if (tableAttribute != null)
            {
                return tableAttribute.Name;
            }

            Type entryType = entry.Entity.GetType();
            string name = entryType.Name;

            if (name.Contains("_"))
            {
                int position = name.IndexOf("_");
                return name.Substring(0, position);
            }

            return entryType.Name;
        }

        private AuditLog GetDeleteAudit(string userToken, DbEntityEntry entry)
        {
            AuditLog log = this.GetAuditRecordForAddOrDelete(userToken, entry);
            log.EventType = "D";
            log.RecordId = this.GetRecordIdFromOriginalValues(entry);
            return log;
        }

        private AuditLog GetAddedAudit(string userToken, DbEntityEntry entry)
        {
            AuditLog log = this.GetAuditRecordForAddOrDelete(userToken, entry);
            log.EventType = "C";
            log.RecordId = this.GetRecordIdFromCurrentValues(entry);
            return log;
        }

        private AuditLog GetAuditRecordForAddOrDelete(string userToken, DbEntityEntry entry)
        {
            return new AuditLog
            {
                Id = Guid.NewGuid(),
                UserToken = userToken,
                EventDateStamp = this.changeTime,
                TableName = this.GetTableName(entry),
                ColumnName = "ALL",
                NewValue = null,
                OriginalValue = null
            };
        }

        private IEnumerable<AuditLog> GetModifiedAudit(string userToken, DbEntityEntry entry)
        {
            var logs = new List<AuditLog>();
            const string eventType = "U";
            string tableName = this.GetTableName(entry);
            string recordId = this.GetRecordIdFromOriginalValues(entry);

            logs.AddRange(
                entry.OriginalValues.PropertyNames
                    .Where(propertyName => entry.OriginalValues.GetValue<object>(propertyName) != entry.CurrentValues.GetValue<object>(propertyName))
                    .Select(propertyName =>
                        new AuditLog
                        {
                            Id = Guid.NewGuid(),
                            UserToken = userToken,
                            EventDateStamp = this.changeTime,
                            EventType = eventType,
                            TableName = tableName,
                            RecordId = recordId,
                            ColumnName = propertyName,
                            OriginalValue = entry.OriginalValues.GetValue<object>(propertyName) == null
                                ? null : entry.OriginalValues.GetValue<object>(propertyName).ToString(),
                            NewValue = entry.CurrentValues.GetValue<object>(propertyName) == null
                                ? null : entry.CurrentValues.GetValue<object>(propertyName).ToString()
                        }));

            return logs;
        }
    }
}
