﻿using System;
using System.Collections.Specialized;
using System.Linq;
using System.Web.Security;

namespace DL.Util.Web.Security.EntityFramework
{
    public class EntityFrameworkMembershipProvider : MembershipProvider
    {
        #region Private Fields

        private string connectionStringName;

        private int maxInvalidPasswordAttempts;

        private int passwordAttemptWindow;

        private int minRequiredNonAlphanumericCharacters;

        private int minRequiredPasswordLength;

        #endregion

        #region Properties

        public override string ApplicationName
        {
            get { return string.Empty; }
            set { this.ApplicationName = string.Empty; }
        }

        public override bool EnablePasswordReset
        {
            get { return false; }
        }

        public override bool EnablePasswordRetrieval
        {
            get { return false; }
        }

        public override bool RequiresQuestionAndAnswer
        {
            get { return false; }
        }

        public override string PasswordStrengthRegularExpression
        {
            get { return string.Empty; }
        }

        public override MembershipPasswordFormat PasswordFormat
        {
            get { return MembershipPasswordFormat.Hashed; }
        }

        public override bool RequiresUniqueEmail
        {
            get { return true; }
        }

        #endregion 

        #region Properties - Field Driven

        public override int MaxInvalidPasswordAttempts
        {
            get { return this.maxInvalidPasswordAttempts; }
        }

        public override int PasswordAttemptWindow
        {
            get { return this.passwordAttemptWindow; }
        }

        public override int MinRequiredNonAlphanumericCharacters
        {
            get { return this.minRequiredNonAlphanumericCharacters; }
        }

        public override int MinRequiredPasswordLength
        {
            get { return this.minRequiredPasswordLength; }
        }

        #endregion

        public override void Initialize(string name, NameValueCollection config)
        {
            if (config == null)
            {
                throw new ArgumentNullException("config");
            }

            base.Initialize(name, config);

            this.InitializeConnectionString(config);
            this.InitializeMaxInvalidPasswordAttempst(config);
            this.InitializeMinRequiredNonAlphanumericCharacters(config);
            this.InitializeMinRequiredPasswordLength(config);
            this.InitializePasswordAttemptWindow(config);
        }

        public override MembershipUser CreateUser(string username, string password, string email, string passwordQuestion, string passwordAnswer, bool isApproved, object providerUserKey, out MembershipCreateStatus status)
        {
            if (string.IsNullOrEmpty(username))
            {
                status = MembershipCreateStatus.InvalidUserName;
                return null;
            }

            if (string.IsNullOrEmpty(password))
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            if (string.IsNullOrEmpty(email))
            {
                status = MembershipCreateStatus.InvalidEmail;
                return null;
            }

            string hashedPassword = HashHelper.HashPassword(password);
            if (hashedPassword.Length > 128)
            {
                status = MembershipCreateStatus.InvalidPassword;
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                if (context.Users.Any(x => x.UserName == username))
                {
                    status = MembershipCreateStatus.DuplicateUserName;
                    return null;
                }

                if (context.Users.Any(x => x.Email == email))
                {
                    status = MembershipCreateStatus.DuplicateEmail;
                    return null;
                }

                var user = new User
                    {
                        UserName = username,
                        Password = hashedPassword,
                        IsApproved = isApproved,
                        Email = email,
                        DateCreated = DateTimeOffset.UtcNow,
                        DatePasswordLastChanged = DateTimeOffset.UtcNow,
                        PasswordFailuresSinceLastSuccess = 0,
                        DateLastLogin = DateTimeOffset.UtcNow,
                        DateLastActivity = DateTimeOffset.UtcNow,
                        DateLastLockedOut = DateTimeOffset.UtcNow,
                        IsLockedOut = false,
                        DateLastPasswordFailure = DateTimeOffset.UtcNow
                    };

                context.Users.Add(user);
                context.SaveChanges();
                status = MembershipCreateStatus.Success;

                return this.GetMembershipUserFromUser(user);
            }
        }

        public override bool ChangePasswordQuestionAndAnswer(string username, string password, string newPasswordQuestion, string newPasswordAnswer)
        {
            throw new NotSupportedException();
        }

        public override string GetPassword(string username, string answer)
        {
            throw new NotSupportedException();
        }

        public override bool ChangePassword(string username, string oldPassword, string newPassword)
        {
            if (string.IsNullOrEmpty(username))
            {
                return false;
            }
            
            if (string.IsNullOrEmpty(oldPassword))
            {
                return false;
            }

            if (string.IsNullOrEmpty(newPassword))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.UserName == username);
                if (user == null)
                {
                    return false;
                }

                string hashedPassword = user.Password;
                bool verificationSucceeded = hashedPassword != null
                                              && HashHelper.VerifyHashedPassword(hashedPassword, oldPassword);
                if (verificationSucceeded)
                {
                    user.PasswordFailuresSinceLastSuccess = 0;
                }
                else
                {
                    user.DateLastPasswordFailure = DateTimeOffset.UtcNow;

                    int failures = user.PasswordFailuresSinceLastSuccess;
                    
                    if (failures < this.MaxInvalidPasswordAttempts)
                    {
                        user.PasswordFailuresSinceLastSuccess += 1;  
                    }
                    else
                    {
                        user.DateLastLockedOut = DateTimeOffset.UtcNow;
                        user.IsLockedOut = true;
                    }

                    context.SaveChanges();
                    return false;
                }

                string newHashedPassword = HashHelper.HashPassword(newPassword);
                if (newHashedPassword.Length > 128)
                {
                    return false;
                }

                user.Password = newHashedPassword;
                user.DatePasswordLastChanged = DateTimeOffset.UtcNow;
                context.SaveChanges();
                return true;
            }
        }

        public override string ResetPassword(string username, string answer)
        {
            throw new NotSupportedException();
        }

        public override void UpdateUser(MembershipUser user)
        {
            throw new NotImplementedException();
        }

        public override bool ValidateUser(string username, string password)
        {
            if (string.IsNullOrEmpty(username))
            {
                return false;
            }

            if (string.IsNullOrEmpty(password))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.UserName == username);

                if (user == null)
                {
                    return false;
                }

                if (!user.IsApproved || user.IsLockedOut)
                {
                    return false;
                }

                string hashedPassword = user.Password;
                bool verificationSucceeded = hashedPassword != null
                                             && HashHelper.VerifyHashedPassword(hashedPassword, password);
                if (verificationSucceeded)
                {
                    user.PasswordFailuresSinceLastSuccess = 0;
                    user.DateLastLogin = DateTimeOffset.UtcNow;
                    user.DateLastActivity = DateTimeOffset.UtcNow;
                    context.SaveChanges();
                    return true;
                }
                
                user.DateLastPasswordFailure = DateTimeOffset.UtcNow;
                int failures = user.PasswordFailuresSinceLastSuccess;
                    
                if (failures < this.MaxInvalidPasswordAttempts)
                {
                    user.PasswordFailuresSinceLastSuccess += 1;    
                }
                else
                {
                    user.DateLastLockedOut = DateTimeOffset.UtcNow;
                    user.IsLockedOut = true;
                }

                context.SaveChanges();
                return false;
            }
        }

        public override bool UnlockUser(string userName)
        {
            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.UserName == userName);
                
                if (user != null)
                {
                    user.IsLockedOut = false;
                    user.PasswordFailuresSinceLastSuccess = 0;
                    context.SaveChanges();
                    return true;
                }
                
                return false;
            }
        }

        public override MembershipUser GetUser(object providerUserKey, bool userIsOnline)
        {
            if (!(providerUserKey is int))
            {
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.Find(providerUserKey);

                if (user != null)
                {
                    if (userIsOnline)
                    {
                        user.DateLastActivity = DateTimeOffset.UtcNow;
                        context.SaveChanges();
                    }

                    return this.GetMembershipUserFromUser(user);
                }

                return null;
            }
        }

        public override MembershipUser GetUser(string username, bool userIsOnline)
        {
            if (string.IsNullOrEmpty(username))
            {
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.UserName == username);

                if (user != null)
                {
                    if (userIsOnline)
                    {
                        user.DateLastActivity = DateTimeOffset.UtcNow;
                        context.SaveChanges();
                    }

                    return this.GetMembershipUserFromUser(user);
                }
                
                return null;
            }
        }

        public override string GetUserNameByEmail(string email)
        {
            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.Email == email);
                return user != null ? user.UserName : string.Empty;
            }
        }

        public override bool DeleteUser(string username, bool deleteAllRelatedData)
        {
            if (string.IsNullOrEmpty(username))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.SingleOrDefault(x => x.UserName == username);
                
                if (user != null)
                {
                    context.Users.Remove(user);
                    context.SaveChanges();
                    return true;
                }
                
                return false;
            }
        }

        public override MembershipUserCollection GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            var membershipUsers = new MembershipUserCollection();

            using (var context = this.GetProviderContext())
            {
                totalRecords = context.Users.Count();
                
                IQueryable<User> users = context.Users
                    .OrderBy(Usrn => Usrn.UserName)
                    .Skip(pageIndex * pageSize)
                    .Take(pageSize);

                foreach (User user in users)
                {
                    membershipUsers.Add(this.GetMembershipUserFromUser(user));
                }
            }

            return membershipUsers;
        }

        public override int GetNumberOfUsersOnline()
        {
            DateTime dateActive 
                = DateTime.UtcNow.Subtract(TimeSpan.FromMinutes(Convert.ToDouble(Membership.UserIsOnlineTimeWindow)));
            
            using (var context = this.GetProviderContext())
            {
                return context.Users.Count(x => x.DateLastActivity > dateActive);
            }
        }

        public override MembershipUserCollection FindUsersByName(string usernameToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            var membershipUsers = new MembershipUserCollection();
            
            using (var context = this.GetProviderContext())
            {
                totalRecords = context.Users.Count(Usr => Usr.UserName == usernameToMatch);
                IQueryable<User> users = context.Users
                    .Where(Usr => Usr.UserName == usernameToMatch)
                    .OrderBy(Usrn => Usrn.UserName)
                    .Skip(pageIndex * pageSize)
                    .Take(pageSize);

                foreach (User user in users)
                {
                    membershipUsers.Add(this.GetMembershipUserFromUser(user));
                }
            }

            return membershipUsers;
        }

        public override MembershipUserCollection FindUsersByEmail(string emailToMatch, int pageIndex, int pageSize, out int totalRecords)
        {
            var membershipUsers = new MembershipUserCollection();

            using (var context = this.GetProviderContext())
            {
                totalRecords = context.Users.Count(Usr => Usr.Email == emailToMatch);
                IQueryable<User> users = context.Users
                    .Where(x => x.Email == emailToMatch)
                    .OrderBy(Usrn => Usrn.UserName)
                    .Skip(pageIndex * pageSize)
                    .Take(pageSize);
                
                foreach (User user in users)
                {
                    membershipUsers.Add(this.GetMembershipUserFromUser(user));
                }
            }

            return membershipUsers;
        }

        #region Private Methods

        private MembershipUser GetMembershipUserFromUser(User user)
        {
            return new MembershipUser(
                    Membership.Provider.Name,
                    user.UserName,
                    user.Id,
                    user.Email,
                    null,
                    null,
                    user.IsApproved,
                    user.IsLockedOut,
                    user.DateCreated.UtcDateTime,
                    user.DateLastLogin.Value.UtcDateTime,
                    user.DateLastActivity.Value.UtcDateTime,
                    user.DatePasswordLastChanged.UtcDateTime,
                    user.DateLastLockedOut.Value.UtcDateTime);
        }

        private ProviderContext GetProviderContext()
        {
            return new ProviderContext(this.connectionStringName);
        }

        private void InitializeConnectionString(NameValueCollection config)
        {
            const string connectionStringNameKey = "connectionStringName";
            
            if (config.AllKeys.Contains(connectionStringNameKey))
            {
                this.connectionStringName = config[connectionStringNameKey];
            }
        }

        private void InitializeMaxInvalidPasswordAttempst(NameValueCollection config)
        {
            const string maxInvalidPasswordAttemptsKey = "maxInvalidPasswordAttempts";

            if (config.AllKeys.Contains(maxInvalidPasswordAttemptsKey))
            {
                string value = config[maxInvalidPasswordAttemptsKey];
                this.maxInvalidPasswordAttempts = int.Parse(value);
            }
            else
            {
                this.maxInvalidPasswordAttempts = 5;
            }
        }
        
        private void InitializePasswordAttemptWindow(NameValueCollection config)
        {
            const string passwordAttemptWindowKey = "passwordAttemptWindow";

            if (config.AllKeys.Contains(passwordAttemptWindowKey))
            {
                string value = config[passwordAttemptWindowKey];
                this.passwordAttemptWindow = int.Parse(value);
            }
            else
            {
                this.passwordAttemptWindow = 10;
            }
        }

        private void InitializeMinRequiredNonAlphanumericCharacters(NameValueCollection config)
        {
            const string minRequiredNonAlphanumericCharactersKey = "minRequiredNonalphanumericCharacters";

            if (config.AllKeys.Contains(minRequiredNonAlphanumericCharactersKey))
            {
                string value = config[minRequiredNonAlphanumericCharactersKey];
                this.minRequiredNonAlphanumericCharacters = int.Parse(value);
            }
            else
            {
                this.minRequiredNonAlphanumericCharacters = 0;
            }
        }

        private void InitializeMinRequiredPasswordLength(NameValueCollection config)
        {
            const string minRequiredPasswordLengthKey = "minRequiredPasswordLength";

            if (config.AllKeys.Contains(minRequiredPasswordLengthKey))
            {
                string value = config[minRequiredPasswordLengthKey];
                this.minRequiredPasswordLength = int.Parse(value);
            }
            else
            {
                this.minRequiredPasswordLength = 6;
            }
        }

        #endregion
    }
}
