﻿using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Web.Security;

namespace DL.Util.Web.Security.EntityFramework
{
    public class EntityFrameworkRoleProvider : RoleProvider
    {
        private string connectionStringName;

        public override string ApplicationName
        {
            get { return string.Empty; }
            set { this.ApplicationName = string.Empty; }
        }

        public override void Initialize(string name, NameValueCollection config)
        {
            const string connectionStringNameKey = "connectionStringName";

            if (config.AllKeys.Contains(connectionStringNameKey))
            {
                this.connectionStringName = config[connectionStringNameKey];
            }

            base.Initialize(name, config);
        }

        public override bool IsUserInRole(string username, string roleName)
        {
            if (string.IsNullOrEmpty(username))
            {
                return false;
            }
            
            if (string.IsNullOrEmpty(roleName))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.Include("Roles").SingleOrDefault(x => x.UserName == username);
                if (user == null)
                {
                    return false;
                }

                Role role = context.Roles.SingleOrDefault(x => x.Name == roleName);
                if (role == null)
                {
                    return false;
                }

                return user.Roles.Contains(role);
            }
        }

        public override string[] GetRolesForUser(string username)
        {
            if (string.IsNullOrEmpty(username))
            {
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                User user = context.Users.Include("Roles").SingleOrDefault(x => x.UserName == username);
                return user != null 
                    ? user.Roles.Select(x => x.Name).ToArray() 
                    : null;
            }
        }

        public override void CreateRole(string roleName)
        {
            if (!string.IsNullOrEmpty(roleName))
            {
                using (var context = this.GetProviderContext())
                {
                    Role role = context.Roles.FirstOrDefault(x => x.Name == roleName);
                    if (role == null)
                    {
                        var newRole = new Role
                        {
                            Name = roleName
                        };
                        context.Roles.Add(newRole);
                        context.SaveChanges();
                    }
                }
            }
        }

        public override bool DeleteRole(string roleName, bool throwOnPopulatedRole)
        {
            if (string.IsNullOrEmpty(roleName))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                Role role = context.Roles.Include("Users").SingleOrDefault(x => x.Name == roleName);
                
                if (role == null)
                {
                    return false;
                }
                
                if (throwOnPopulatedRole && role.Users.Any())
                {
                    return false;
                }

                role.Users.Clear();
                context.Roles.Remove(role);
                context.SaveChanges();
                return true;
            }
        }

        public override bool RoleExists(string roleName)
        {
            if (string.IsNullOrEmpty(roleName))
            {
                return false;
            }

            using (var context = this.GetProviderContext())
            {
                return context.Roles.Any(x => x.Name == roleName);
            }
        }

        public override void AddUsersToRoles(string[] usernames, string[] roleNames)
        {
            using (var context = this.GetProviderContext())
            {
                List<User> users = context.Users.Include("Roles").Where(x => usernames.Contains(x.UserName)).ToList();
                List<Role> roles = context.Roles.Where(x => roleNames.Contains(x.Name)).ToList();

                foreach (User user in users)
                {
                    User localUser = user;
                    foreach (Role role in roles.Where(r => !localUser.Roles.Contains(r)))
                    {
                        user.Roles.Add(role);
                    }
                }

                context.SaveChanges();
            }
        }

        public override void RemoveUsersFromRoles(string[] usernames, string[] roleNames)
        {
            using (var context = this.GetProviderContext())
            {
                List<User> users = context.Users.Include("Roles").Where(x => usernames.Contains(x.UserName)).ToList();
                List<Role> roles = context.Roles.Where(x => roleNames.Contains(x.Name)).ToList();

                foreach (User user in users)
                {
                    User localUser = user;
                    foreach (Role role in roles.Where(r => localUser.Roles.Contains(r)))
                    {
                        user.Roles.Remove(role);
                    }
                }

                context.SaveChanges();
            }
        }

        public override string[] GetUsersInRole(string roleName)
        {
            if (string.IsNullOrEmpty(roleName))
            {
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                Role role = context.Roles.Include("Users").SingleOrDefault(x => x.Name == roleName);

                return role != null 
                    ? role.Users.Select(x => x.UserName).ToArray() 
                    : null;
            }
        }

        public override string[] GetAllRoles()
        {
            using (var context = this.GetProviderContext())
            {
                return context.Roles.Select(x => x.Name).ToArray();
            }
        }

        public override string[] FindUsersInRole(string roleName, string usernameToMatch)
        {
            if (string.IsNullOrEmpty(roleName))
            {
                return null;
            }

            if (string.IsNullOrEmpty(usernameToMatch))
            {
                return null;
            }

            using (var context = this.GetProviderContext())
            {
                return context.Users.Include("Roles")
                    .Where(x => x.Roles.Any(r => r.Name == roleName)
                        && x.UserName.Contains(usernameToMatch))
                    .Select(x => x.UserName)
                    .ToArray();
            }
        }

        private ProviderContext GetProviderContext()
        {
            return new ProviderContext(this.connectionStringName);
        }
    }
}
