﻿using System;
using System.Linq;
using System.Xml.Linq;

namespace DL.Core.Web.Rss
{
    /// <summary>
    /// Class which parses RSS feeds.
    /// </summary>
    public class RssClient
    {
        /// <summary>
        /// Parses an RSS feed.
        /// </summary>
        /// <param name="url">URL of the feed to parse.</param>
        /// <returns>Object representation of the parsed RSS feed.</returns>
        public Feed GetRssFeed(string url)
        {
            var feed = new Feed();
            var document = XDocument.Load(url);

            XNamespace namespaceContent = "http://purl.org/rss/1.0/modules/content/";
            XNamespace namespaceWfw = "http://wellformedweb.org/CommentAPI/";
            XNamespace namespaceDc = "http://purl.org/dc/elements/1.1/";
            XNamespace namespaceSy = "http://purl.org/rss/1.0/modules/syndication/";
            XNamespace namespaceSlash = "http://purl.org/rss/1.0/modules/slash/";

            XElement channel = document.Descendants("channel").FirstOrDefault();

            if (channel.Element("title") != null)
            {
                feed.Title = channel.Element("title").Value;
            }

            if (channel.Element("link") != null)
            {
                feed.Link = channel.Element("link").Value;
            }

            if (channel.Element("description") != null)
            {
                feed.Description = channel.Element("description").Value;
            }

            if (channel.Element("lastBuildDate") != null)
            {
                feed.LastBuildDate = DateTime.Parse(channel.Element("lastBuildDate").Value);
            }

            if (channel.Element("language") != null)
            {
                feed.Language = channel.Element("language").Value;
            }

            if (channel.Element("generator") != null)
            {
                feed.Generator = channel.Element("generator").Value;
            }

            if (channel.Element("image") != null)
            {
                XElement imageElement = channel.Element("image");

                if (imageElement.Element("url") != null)
                {
                    feed.Image.Url = channel.Element("image").Element("url").Value;
                }

                if (imageElement.Element("title") != null)
                {
                    feed.Image.Title = channel.Element("image").Element("title").Value;
                }

                if (imageElement.Element("link") != null)
                {
                    feed.Image.Link = channel.Element("image").Element("link").Value;
                }
            }           

            if (channel.Element(namespaceSy + "updatePeriod") != null)
            {
                feed.UpdatePeriod = channel.Element(namespaceSy + "updatePeriod").Value;
            }

            if (channel.Element(namespaceSy + "updateFrequency") != null)
            {
                feed.UpdateFrequency = channel.Element(namespaceSy + "updateFrequency").Value;
            }

            foreach (XElement itemElement in channel.Descendants("item"))
            {
                var item = new Item();

                if (itemElement.Element("title") != null)
                {
                    item.Title = itemElement.Element("title").Value;
                }

                if (itemElement.Element("link") != null)
                {
                    item.Link = itemElement.Element("link").Value;
                }

                if (itemElement.Element("description") != null)
                {
                    item.Description = itemElement.Element("description").Value;
                }

                if (itemElement.Element("comments") != null)
                {
                    item.CommentsLink = itemElement.Element("comments").Value;
                }

                if (itemElement.Element(namespaceSlash + "comments") != null)
                {
                    item.NumberOfComments = Convert.ToInt32(itemElement.Element(namespaceSlash + "comments").Value);
                }

                if (itemElement.Element(namespaceWfw + "commentRss") != null)
                {
                    item.CommentRss = itemElement.Element(namespaceWfw + "commentRss").Value;
                }

                if (itemElement.Element("pubDate") != null)
                {
                    item.PublicationDate = Convert.ToDateTime(itemElement.Element("pubDate").Value);
                }

                if (itemElement.Element("guid") != null)
                {
                    item.Guid = itemElement.Element("guid").Value;
                }

                if (itemElement.Element(namespaceDc + "creator") != null)
                {
                    item.Author = itemElement.Element(namespaceDc + "creator").Value;
                }

                if (itemElement.Element(namespaceContent + "encoded") != null)
                {
                    item.EncodedContent = itemElement.Element(namespaceContent + "encoded").Value;
                }

                if (itemElement.Elements("category") != null)
                {
                    item.Tags = (from category in itemElement.Elements("category")
                                    orderby category.Value
                                    select category.Value).ToList();
                }

                feed.Items.Add(item);
            }

            return feed;
        }
    }
}
