﻿using System;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace DL.Core.Web
{
    /// <summary>
    /// Class which contains web utility methods.
    /// </summary>
    public static class WebUtil
    {
        /// <summary>
        /// Gets the current user of the web application. 
        /// Uses multiple methods to find user.
        /// </summary>
        /// <param name="context">HTTP Context object.</param>
        /// <returns>Current user of the web application.</returns>
        public static string GetCurrentUser(HttpContext context)
        {
            string contextUser = context.User.Identity.Name;
            if (!string.IsNullOrEmpty(contextUser))
            {
                return contextUser;
            }

            string logonUser = GetRequestLogonUser(context.Request);
            if (!string.IsNullOrEmpty(logonUser))
            {
                return logonUser;
            }

            string remoteUser = GetRequestRemoteUser(context.Request);
            return !string.IsNullOrEmpty(remoteUser) ? remoteUser : string.Empty;
        }

        /// <summary>
        /// Gets the selected text of a control inheriting from ListControl.
        /// </summary>
        /// <param name="control">
        /// Control which inherits from ListControl. Ex: BulletedList,
        /// CheckBoxList, DropDownList, ListBox, RadioButtonList
        /// </param>
        /// <returns>
        /// Text from the selected item or empty string if selected item is null.
        /// </returns>
        public static string GetListControlSelectedText(ListControl control)
        {
            if (control.SelectedItem == null
                || string.IsNullOrEmpty(control.SelectedItem.Text))
            {
                return string.Empty;
            }

            return control.SelectedItem.Text;
        }

        /// <summary>
        /// Gets the HTTP Request logon user server variable.
        /// </summary>
        /// <param name="request">HttpRequest object.</param>
        /// <returns>Logon user server variable of the given HttpRequest object.</returns>
        public static string GetRequestLogonUser(HttpRequest request)
        {
            string user = request.ServerVariables["LOGON_USER"];
            return string.IsNullOrEmpty(user) ? string.Empty : user;
        }

        /// <summary>
        /// Gets the HTTP Request remote user server variable.
        /// </summary>
        /// <param name="request">HttpRequest object.</param>
        /// <returns>Remote user server variable of the given HttpRequest object.</returns>
        public static string GetRequestRemoteUser(HttpRequest request)
        {
            string user = request.ServerVariables["REMOTE_USER"];
            return string.IsNullOrEmpty(user) ? string.Empty : user;
        }

        /// <summary>
        /// Web version if File.Exists. Uses HTTP request and response to 
        /// determine if the given file exists.
        /// </summary>
        /// <param name="url">URL of file to check.</param>
        /// <returns>True if exists, otherwise false.</returns>
        public static bool IfWebFileExists(string url)
        {
            try
            {
                var request = WebRequest.Create(new Uri(url));
                request.Method = "HEAD";

                using (var response = (HttpWebResponse)request.GetResponse())
                {
                    if (response != null 
                        && response.StatusCode == HttpStatusCode.OK)
                    {
                        return true;
                    }
                    //// todo: Other HTTP response - check StatusCode and handle
                }
            }
            catch (WebException ex)
            {
                HttpWebResponse webResponse = (HttpWebResponse)ex.Response;

                if (webResponse.StatusCode == HttpStatusCode.NotFound)
                {
                    // file doesn't exist
                    return false;
                }
                //// todo: handle other status codes
            }

            return false;
        }

        /// <summary>
        /// Determines whether or not a query string key is valid.
        /// </summary>
        /// <param name="request">Current HttpRequest object.</param>
        /// <param name="key">Key of the query string to validate.</param>
        /// <returns>True if valid, otherwise false.</returns>
        public static bool IsQueryStringValid(HttpRequest request, string key)
        {
            if (request.QueryString[key] != null
                && !string.IsNullOrEmpty(request.QueryString[key]))
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Finds a control by ID.
        /// </summary>
        /// <param name="root">System.Web.UI.Control object to use as the root of the find control search.</param>
        /// <param name="idToFind">ID of the control you wish to find within the root control.</param>
        /// <returns>The Control object with the given ID or null if not found.</returns>
        public static Control RecursiveFindControl(Control root, string idToFind)
        {
            if (root.ID == idToFind)
            {
                return root;
            }

            return (from Control control in root.Controls 
                    select RecursiveFindControl(control, idToFind))
                    .FirstOrDefault(childControl => childControl != null);
        }
    }
}
