﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace DL.Core.Enums
{
    /// <summary>
    /// Class which contains enum utility methods.
    /// </summary>
    public static class EnumUtil
    {
        ///// <summary>
        ///// Gets a list of the items within an enumeration type. Values will be filled with 
        ///// the constant given to each enumeration type value and the Display will be filled with
        ///// either the enumeration type value or the ComponentModel DescriptionAttribute.
        ///// </summary>
        ///// <param name="type">Enumeration type to use. Ex: typeof(EnumerationTypeName)</param>
        ///// <returns>A list of ListItemData for the given enumeration type.</returns>
        //public static IEnumerable<ListItemData> ConvertValuesToListItemData(Type type)
        //{
        //    if (type == null)
        //    {
        //        throw new ArgumentNullException("type");
        //    }

        //    if (!type.IsEnum)
        //    {
        //        throw new ArgumentException("The 'type' parameter must contain an Enumeration type.");
        //    }

        //    var returnValues = new List<ListItemData>();
        //    var names = Enum.GetNames(type);

        //    foreach (string name in names)
        //    {
        //        var field = type.GetField(name);
        //        var attribute = field.GetCustomAttributes(typeof(DescriptionAttribute), false);

        //        var item
        //            = new ListItemData
        //            {
        //                Display = attribute.Length > 0 ? ((DescriptionAttribute)attribute[0]).Description : name,
        //                Value = Convert.ToInt32(field.GetRawConstantValue())
        //            };
        //        returnValues.Add(item);
        //    }

        //    return returnValues;
        //}

        public static T GetByByte<T>(byte byteValue)
        {
            if (!IsValidEnumByte<T>(byteValue))
            {
                throw new ArgumentOutOfRangeException("byteValue");
            }

            return (T)Enum.ToObject(typeof(T), byteValue);
        }

        public static byte GetByte(object value)
        {
            return Convert.ToByte(value);
        }

        /// <summary>
        /// Gets an Enumeration value by its associated integer value.
        /// </summary>
        /// <typeparam name="T">Enumeration Type to use. Ex: typeof(EnumerationTypeName)</typeparam>
        /// <param name="integer">Integer value.</param>
        /// <returns>Enumeration value of type T.</returns>
        public static T GetByInteger<T>(int integer)
        {
            if (!IsValidEnumInteger<T>(integer))
            {
                throw new ArgumentOutOfRangeException("integer");
            }

            return (T)Enum.ToObject(typeof(T), integer);
        }

        /// <summary>
        /// Gets the integer value associated with the given enumeration type value.
        /// </summary>
        /// <param name="value">Enumeration type value.</param>
        /// <returns>Integer value.</returns>
        public static int GetInteger(object value)
        {
            return (int)value;
        }

        /// <summary>
        /// Gets the title of the given enumeration type value's integer constant.
        /// </summary>
        /// <typeparam name="T">Enumeration type to use. Ex: typeof(EnumerationName)</typeparam>
        /// <param name="integer">Integer value associated with the enumeration type value.</param>
        /// <returns>String filled with either the enumeration type name or the ComponentModel DescriptionAttribute.</returns>
        public static string GetTitle<T>(int integer)
        {
            T value = GetByInteger<T>(integer);
            return GetTitle(value);
        }

        /// <summary>
        /// Gets the title of the given enumeration type value.
        /// </summary>
        /// <typeparam name="T">Enumeration Type to use. Ex: typeof(EnumerationName)</typeparam>
        /// <param name="value">Enumeration type value.</param>
        /// <returns>String filled with either the enumeration name or the ComponentModel DescriptionAttribute.</returns>
        public static string GetTitle<T>(T value)
        {
            Type enumType = typeof(T);
            string name = Enum.GetName(enumType, value);
            var field = enumType.GetField(name);
            var attribute = field.GetCustomAttributes(typeof(DescriptionAttribute), false);
            return attribute.Length > 0 ? ((DescriptionAttribute)attribute[0]).Description : name;
        }

        /// <summary>
        /// Gets a list of values associated with the given enumeration type.
        /// </summary>
        /// <typeparam name="T">Type of the enumeration</typeparam>
        /// <returns>List of enumeration values.</returns>
        public static IEnumerable<T> GetValues<T>()
        {
            return Enum.GetValues(typeof(T)).Cast<T>();
        }

        /// <summary>
        /// Determines whether a given byte is a byte value within an enumeration.
        /// </summary>
        /// <typeparam name="T">Type of enumeration.</typeparam>
        /// <param name="byteValue">Byte value to check for.</param>
        /// <returns>True if found, otherwise false.</returns>
        public static bool IsValidEnumByte<T>(byte byteValue)
        {
            return Enum.GetValues(typeof(T)).Cast<object>().Any(enumValue => byteValue == Convert.ToByte(enumValue));
        }

        /// <summary>
        /// Determines whether a given integer is an integer value within an enumeration.
        /// </summary>
        /// <typeparam name="T">Type of enumeration.</typeparam>
        /// <param name="integer">Integer to check for.</param>
        /// <returns>True if found, otherwise false.</returns>
        public static bool IsValidEnumInteger<T>(int integer)
        {
            return Enum.GetValues(typeof(T)).Cast<int>().Any(value => value == integer);
        }
    }
}
