﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;

namespace DL.Core.Expressions
{
    public class ExpressionBuilder<T>
    {
        private readonly ParameterExpression parameterExpression;

        private readonly BinaryExpressionBuilder binaryExpressionBuilder;

        private readonly MethodCallExpressionBuilder methodCallExpressionBuilder;

        public ExpressionBuilder()
        {
            this.Filters = new List<ExpressionFilter>();
            this.parameterExpression = Expression.Parameter(typeof(T), "x");

            this.binaryExpressionBuilder = new BinaryExpressionBuilder(this.parameterExpression);
            this.methodCallExpressionBuilder = new MethodCallExpressionBuilder(this.parameterExpression);
        }

        public ExpressionBuilder(IEnumerable<ExpressionFilter> filters)
            : this()
        {
            this.Filters.AddRange(filters);
        }

        public List<ExpressionFilter> Filters { get; set; }

        public Expression<Func<T, bool>> GetExpression()
        {
            if (this.Filters.Count == 0)
            {
                throw new InvalidOperationException("Filters must be setup before this method can be used.");
            }

            Expression expression = this.BuildSinglePartExpression(this.Filters[0]);

            for (int i = 1; i < this.Filters.Count; i++)
            {
                ExpressionFilter currentFilter = this.Filters[i];
                Expression rightExpression = this.BuildSinglePartExpression(currentFilter);

                switch (currentFilter.CompositeType)
                {
                    case ExpressionCompositeType.And:
                        expression = Expression.Add(expression, rightExpression);
                        break;
                    case ExpressionCompositeType.Or:
                        expression = Expression.Or(expression, rightExpression);
                        break;
                    default:
                        throw new ArgumentOutOfRangeException();
                }
            }       
            
            return Expression.Lambda<Func<T, bool>>(expression, this.parameterExpression);
        }

        private Expression BuildSinglePartExpression(ExpressionFilter filter)
        {
            switch (filter.FilterType)
            {
                case ExpressionFilterType.Equals:
                    return this.binaryExpressionBuilder.GetEqualsExpression(filter.PropertyName, filter.FilterValue);
                case ExpressionFilterType.BetweenInclusive:
                    return this.binaryExpressionBuilder.GetBetweenInclusiveExpression(filter.PropertyName, filter.FilterValue, filter.FilterValueTwo);
                case ExpressionFilterType.BetweenExclusive:
                    return this.binaryExpressionBuilder.GetBetweenExclusiveExpression(filter.PropertyName, filter.FilterValue, filter.FilterValueTwo);
                case ExpressionFilterType.LessThan:
                    return this.binaryExpressionBuilder.GetLessThanExpression(filter.PropertyName, filter.FilterValue);
                case ExpressionFilterType.LessThanOrEqual:
                    return this.binaryExpressionBuilder.GetLessThanOrEqualExpression(filter.PropertyName, filter.FilterValue);
                case ExpressionFilterType.GreaterThan:
                    return this.binaryExpressionBuilder.GetGreaterThanExpression(filter.PropertyName, filter.FilterValue);
                case ExpressionFilterType.GreaterThanOrEqual:
                    return this.binaryExpressionBuilder.GetGreaterThanOrEqualExpression(filter.PropertyName, filter.FilterValue);
                case ExpressionFilterType.Contains:
                    return this.methodCallExpressionBuilder.GetContainsExpression(filter.PropertyName, filter.FilterValue.ToString());
                case ExpressionFilterType.StartsWith:
                    return this.methodCallExpressionBuilder.GetStartsWithExpression(filter.PropertyName, filter.FilterValue.ToString());
                case ExpressionFilterType.EndsWith:
                    return this.methodCallExpressionBuilder.GetEndsWithExpression(filter.PropertyName, filter.FilterValue.ToString());
                default:
                    throw new ArgumentOutOfRangeException();
            }
        }
    }
}
